"""empty message

Revision ID: 22033ea27803
Revises: 0abab7881256
Create Date: 2025-03-14 14:24:16.523888

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '22033ea27803'
down_revision: Union[str, None] = '0abab7881256'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('todo', sa.Column('is_deleted', sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('todo', 'is_deleted')
    # ### end Alembic commands ###



from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

# Dynamic database path configuration
import sys
from pathlib import Path
from platformdirs import user_data_dir

if hasattr(sys, 'real_prefix') or (hasattr(sys, 'base_prefix') and sys.prefix != sys.base_prefix):
    data_dir = Path(sys.prefix) / 'todo_data'
else:
    data_dir = Path(user_data_dir('todo-cli'))

data_dir.mkdir(parents=True, exist_ok=True)
db_path = data_dir / 'todo.db'

SQLALCHEMY_DATABASE_URL = f"sqlite:///{db_path}"

engine = create_engine(SQLALCHEMY_DATABASE_URL)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
Base = declarative_base()

# Add this at the bottom of the file after creating engine
from todo_cli import models  # Add this import
Base.metadata.create_all(bind=engine)


"""empty message

Revision ID: 0abab7881256
Revises: 
Create Date: 2025-03-14 10:44:18.801519

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '0abab7881256'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('todo',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('created_at', sa.DateTime(timezone=True), nullable=False),
    sa.Column('updated_at', sa.DateTime(timezone=True), nullable=True),
    sa.Column('title', sa.String(), nullable=True),
    sa.Column('todo_body', sa.String(), nullable=True),
    sa.Column('priority', sa.String(), nullable=True),
    sa.Column('is_in_progress', sa.Boolean(), nullable=True),
    sa.Column('is_completed', sa.Boolean(), nullable=True),
    sa.Column('complete_time', sa.DateTime(timezone=True), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_todo_id'), 'todo', ['id'], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_todo_id'), table_name='todo')
    op.drop_table('todo')
    # ### end Alembic commands ###



# local imports
from datetime import datetime
from todo_cli.database import SessionLocal, engine, Base
from todo_cli.models import Todo, Setting

# Update the existing Alembic imports at the top
from alembic.config import Config
from alembic import command
import os


def run_migrations():
    # Get the package directory path
    package_dir = os.path.dirname(os.path.abspath(__file__))
    alembic_ini_path = os.path.join(package_dir, "alembic.ini")
    alembic_cfg = Config(alembic_ini_path)
    command.upgrade(alembic_cfg, "head")


# third party imports
import typer
from rich import print
from rich.table import Table
from rich.console import Console
from rich.prompt import Prompt
from enum import Enum
from typing import Annotated

app = typer.Typer()
console = Console()


def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


db = next(get_db())


class Priority(str, Enum):
    LOW = "1" or "LOW"
    MEDIUM = "2" or "MEDIUM"
    HIGH = "3" or "HIGH"


class InProgress(str, Enum):
    TRUE = "1"
    FALSE = "2"


@app.command(help="Display a list of todos that are not completed.")
def list():
    """
    Display a list of todos that are not completed.
    """
    print("[bold white]Todos List :[/bold white]")
    todos = (
        db.query(Todo).filter(Todo.is_completed != True, Todo.is_deleted != True).all()
    )
    table = Table("Id", "Title", "Body", "Priority", "In Progress")
    for todo in todos:
        row_style = "green" if todo.is_in_progress else "white"
        if todo.priority == "LOW":
            priority_color = "yellow"
        elif todo.priority == "MEDIUM":
            priority_color = "blue"
        else:
            priority_color = "purple"
        table.add_row(
            str(todo.id),
            todo.title,
            todo.todo_body,
            f"[{priority_color}]{todo.priority}[/{priority_color}]",
            ":heavy_check_mark:" if todo.is_in_progress else ":x:",
            style=row_style,
        )
    table.caption = "[red]in progress todo's are in green[/red]"
    console.print(table)


@app.command(help="Getting list of all todos (new, in progress, completed)")
def all():
    """
    list_all : Getting list of all todos (new, in progress, completed)
    """
    todos = db.query(Todo).filter(Todo.is_deleted != True).all()
    table = Table("Id", "Title", "Body", "Priority", "In Progress", "Completed")
    for todo in todos:
        if todo.is_in_progress:
            row_style = "green"
        elif todo.is_completed:
            row_style = "yellow"
        else:
            row_style = "white"
        if todo.priority == "LOW":
            priority_color = "yellow"
        elif todo.priority == "MEDIUM":
            priority_color = "blue"
        else:
            priority_color = "purple"

        table.add_row(
            str(todo.id),
            todo.title,
            todo.todo_body,
            f"[{priority_color}]{todo.priority}[/{priority_color}]",
            ":heavy_check_mark:" if todo.is_in_progress else ":x:",
            ":heavy_check_mark:" if todo.is_completed else ":x:",
            style=row_style,
        )
    table.caption = "[green]in progress todos are in green[/green] \n[red]completed todos are in red[/red]"
    console.print(table)


@app.command(help="Adding a todo")
def add():
    """
    Add a new todo to the database.
    """
    title = Prompt.ask("[bold]Title of the todo[/bold]")
    body = Prompt.ask("[bold]Body of the todo[/bold]")

    while True:
        priority = Prompt.ask(
            "[bold]Priority of the todo (1. Low, 2. Medium, 3. High)[/bold]",
        )
        if priority in [p.value for p in Priority]:
            priority = Priority(priority).name
            break
        print("[bold red]Invalid priority! Please enter 1, 2, or 3.[/bold red]")

    while True:
        is_in_progress = Prompt.ask(
            "[bold]Is the todo in progress? (1. True, 2. False)[/bold]"
        )
        if is_in_progress in [s.value for s in InProgress]:
            break
        print("[bold red]Invalid status! Please enter 1 or 2.[/bold red]")

    todo = Todo(
        created_at=datetime.now(),
        title=title,
        todo_body=body,
        priority=priority,
        is_in_progress=(is_in_progress == InProgress.TRUE.value),
        is_completed=False,
    )

    # Add to the database
    db.add(todo)
    db.commit()
    print("[bold green]Todo added successfully![/bold green]")
    list()


@app.command(name="progress", help="Updating a todo that todo is 'in progress'")
def add_to_in_progress(
    todo_id: int,
):
    todo = db.query(Todo).filter(Todo.id == todo_id).first()
    if not todo:
        print("[red bold]Todo not found![/red bold]")
        return
    list()

    while True:
        is_in_progress = (
            Prompt.ask("[bold green]Is todo in progress (1.True/0.False)?[/bold green]")
            .strip()
            .lower()
        )
        if is_in_progress in ["1", "0"]:
            todo.is_in_progress = is_in_progress == "true"
            break
        print("[bold red]Invalid input! Please enter 'True' or 'False'.[/bold red]")

    todo.is_in_progress = bool(int(is_in_progress))
    db.commit()
    print("[green bold]Todo added to in progress todos successfully![/green bold]")
    list()


@app.command(name="completed")
def get_in_completed(todo_id: int):
    todo = db.query(Todo).filter(Todo.id == todo_id).first()
    if not todo:
        print("[red bold]Todo not found![/red bold]")
        return
    while True:
        is_completed = (
            Prompt.ask(
                "[bold yellow]Is todo completed (1.True / 0.False)?[/bold yellow]"
            )
            .strip()
            .lower()
        )
        if int(is_completed) == 1:
            todo.is_completed = True
            todo.is_in_progress = False
            db.commit()
            break
        print("[bold red]Invalid input! Please enter 'True' or 'False'.[/bold red]")

        print("[green bold]Todo added to completed todos successfully![/green bold]")
        list()


@app.command(help="Delete a todo from app")
def delete(todo_id: int):
    todo = db.query(Todo).filter(Todo.id == todo_id).first()
    if not todo:
        print("[red bold]Todo not found![/red bold]")
        return
    todo.is_deleted = True
    db.commit()
    print("[green bold] Todo deleted successfully [/green bold]")
    list()


@app.command(help="Delete al todo from app")
def delete_all(
    force: Annotated[
        bool, typer.Option(prompt="Are you sure you want to delete all todos?")
    ],
):
    todos = db.query(Todo).all()
    if force:
        for todo in todos:
            todo.is_deleted = True
        db.commit()
        print("[green bold] Todos deleted successfully [/green bold]")
        list()


@app.command(help="Get history off all todos (include deleted todos)")
def history():
    todos = db.query(Todo).all()
    table = Table("Id", "Title", "Body", "Priority", "In Progress", "Completed")
    for todo in todos:
        if todo.is_in_progress:
            row_style = "green"
        elif todo.is_completed:
            row_style = "yellow"
        else:
            row_style = "white"
        if todo.priority == "LOW":
            priority_color = "yellow"
        elif todo.priority == "MEDIUM":
            priority_color = "blue"
        else:
            priority_color = "purple"

        table.add_row(
            str(todo.id),
            todo.title,
            todo.todo_body,
            f"[{priority_color}]{todo.priority}[/{priority_color}]",
            ":heavy_check_mark:" if todo.is_in_progress else ":x:",
            ":heavy_check_mark:" if todo.is_completed else ":x:",
            style=row_style,
        )
    table.caption = "[green]in progress todos are in green[/green] \n[red]completed todos are in red[/red]"
    console.print(table)


def main():
    todo = db.query(Setting).first()
    if todo.is_database_migrated is None or todo.is_database_migrated is False:
        run_migrations()
    print("CCC")
    app()



from sqlalchemy import Column, Integer, String, Boolean, DateTime
from .database import Base

class Todo(Base):
    __tablename__ = "todo"
    
    id = Column(Integer, primary_key=True, index=True)
    title = Column(String)
    todo_body = Column(String)
    created_at = Column(DateTime(timezone=True), nullable=False)
    updated_at = Column(DateTime(timezone=True))
    priority = Column(String)
    is_in_progress = Column(Boolean, default=False)
    is_completed = Column(Boolean, default=False)
    complete_time = Column(DateTime(timezone=True))
    is_deleted = Column(Boolean, default=False)

class Setting(Base):
    __tablename__ = "setting"
    id = Column(Integer, primary_key=True, index=True)
    is_database_migrated = Column(Boolean, default=False)



from setuptools import setup

setup(
    name="rich-todo",
    version="0.1.2",
    packages=["todo_cli"],
    package_dir={"todo_cli": "todo_cli"},
    include_package_data=True,
    install_requires=["sqlalchemy", "alembic", "typer", "rich", "platformdirs"],
    entry_points={
        "console_scripts": [
            "todo=todo_cli.main:app",
        ],
    },
    author="Mehran Mirzaei",
    author_email="mehranmirzaeiv@gmail.com",
    description="A CLI tool for managing todos directly from the terminal",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="todo, cli, productivity",
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_data={"todo_cli": ["alembic.ini", "alembic/*", "alembic/versions/*"]},
)
